
%  Copyright (c) 2013
%      Oliver van Kaick <ovankaic@gmail.com>

%
% Build terms for graph cuts labeling of a point cloud
%
% Input:
%   - points: <n x 3> matrix of points
%   - gr: neighborhood relations of points, as returned by build_graph
%   - seeds: segment seeds
%
% Output:
%   - datacost term
%   - smoothness term
%
function [datacost, smoothness] = build_terms_small(points, gr, clusters, dist)

    % Set data cost
    n = size(points, 1);
    l = length(clusters);
    datacost = zeros(l, n);
    for i = 1:length(clusters)
        d = dist(i, :);
        datacost(i, :) = (d.^1.5) ./ max(d.^1.5);
    end
    for i = 1:length(clusters)
        sel1 = clusters{i};
        mx = max(dist(i, sel1));
        sel2 = find(dist(i, :) > (mx*0.5));
        sel = intersect(sel1, sel2);
        datacost(:, sel) = 10;
    end
    for i = 1:length(clusters)
        datacost(i, clusters{i}) = 0;
        sel1 = clusters{i};
        mx = max(dist(i, sel1));
        sel = find(dist(i, :) < (mx*0.2));
        datacost(i, sel) = 0;
    end

    % Set smoothness cost 
    % Simple Potts model
    smoothness = zeros(l, l);
    for i = 1:l
        for j = 1:l
            smoothness(i, j) = (i ~= j);
        end
    end
end
