
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%% compute visibility between a pair of patches

%%% input:
% points: 3D point set
% normals: 3D normals corresponding to points
% patch1: candidate points of first patch
% patch2: candidate points of second path
% gr: KNN graph
% outliers: outlier points

%%% output:
% los: number of cast rays that hit the correct target
% ilos: number of cast rays that didn't hit the correct target


function [los, ilos] = patch_los_pair(points, normals, patch1, patch2, gr, outliers)


N = size(points,1);

splatRadiuses = point_cloud_estimate_radiuses(points, normals);
splatRadiusesSquare = splatRadiuses.^2; % useful later on

los = 0;
ilos = 0;

for k=1:length(patch1)

    src = patch1(k);
    srcp = points(src,:);
    srcn = normals(src,:);

    repNormal = repmat(srcn, [N,1]);
    repSource = repmat(srcp, [N,1]);
    faceFronters = sum((points - repSource) .* repNormal, 2) < 0; % all the points that are in front of this point
    
    
    for l=1:length(patch2)
        
        tgt = patch2(l);

        hit_point = cast_ray(points, normals, splatRadiusesSquare, faceFronters, src, tgt, gr, outliers);
        
        if(hit_point == -1)
            [dists, path, succ] = st_shortest_path(points, gr, src, tgt);
            if(succ == 1)
                valid = check_geodesic_path(points, normals, src, tgt, path);
                if(valid)
                    hit_point = tgt;
                end
            end
        end
        
        if(hit_point ~= tgt && hit_point ~= -1)
            ints = intersect(gr{tgt}, gr{hit_point});
            if(~isempty(ints))
                [dists, path, succ] = st_shortest_path(points, gr, hit_point, tgt);
                if(succ == 1)
                    valid = check_geodesic_path(points, normals, hit_point, tgt, path);
                    if(valid)
                        hit_point = tgt;
                    end
                end
            end
        end
        
        if(hit_point == tgt)
            los = los + 1;
        else
            ilos = ilos + 1;
        end
    end
    
end



end



