%
% This function reads an obj file and returns a mesh structure
%
% function [M, status] = mesh_read_obj(filename)
%
% Input -
%   - filename: name of obj file to load
%
% Output -
%   - M: triangle mesh: M.vertices(i, :) represents the 3D coordinates
%   of vertex 'i', while M.faces(i, :) contains the indices of the three
%   vertices that compose face 'i'. If the mesh also has color
%   attributes, they are stored in M.FaceVertexCData
%   - status: this variable is 0 if the file was succesfuly opened, or 1
%   otherwise
%
%   The input mesh is similar to that read by the mesh_read_smf
%   function, but the way of specifying colors changes.
%
% See also mesh_read, mesh_read_smf
%
function P = read_obj(filename)
%
% Copyright (c) 2008, 2009, 2010 Oliver van Kaick <ovankaic@cs.sfu.ca>
%

    % Open file
    fid = fopen(filename);
    status = 0;
    if fid == -1
        disp(['ERROR: could not open file "' filename '"']);
        M = [];
        status = 1;
        return;
    end

    % Read content
    P = [];
    P.n = [];
    P.v = [];
    while(feof(fid) ~= 1)
        line = '';
        line = fgetl(fid);
        % A line needs at least two characters to be meaningful
        if length(line) <= 1
            continue;
        end
        if (line(1) == 'v') && (line(2) == ' ')
            line = line(3:length(line));
            P.v(end+1, :) = sscanf(line, '%f %f %f');
%        elseif (line(1) == 'f') && (line(2) == ' ')
%            line = line(3:length(line));
%            temp = sscanf(line, '%d//%d %d//%d %d//%d');
%            if length(temp) < 6
%                F(fnum, :) = sscanf(line, '%d %d %d');
%            else
%                F(fnum, :) = temp([1 3 5]);
%            end
%            fnum = fnum + 1;
        elseif (line(1) == 'v') && (line(2) == 'n')
            line = line(4:length(line));
            P.n(end+1, :) = sscanf(line, '%f %f %f');
        end
    end

    %X1 = [inf inf inf];
    %for i = 1:vnum
    %    for j = 1:length(X1(:,1))
    %        if X(i,:) == X1(j,:)
    %            break;
    %        end
    %    end
    %    if j == length(X1(:,1));
    %        X1 = [X1;X(i,:)];
    %    end
    %end
    %X = X1(2:length(X1(:,1)),:);

    % Close file
    fclose(fid);
end
